import { mkdirSync, writeFileSync, statSync, readdirSync } from 'fs';
import { join, dirname } from 'path';
import { defineNuxtModule, createResolver } from '@nuxt/kit';

var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var __privateMethod = (obj, member, method) => {
  __accessCheck(obj, member, "access private method");
  return method;
};
var _urls, _basePath, _generateUrls, generateUrls_fn;
class SiteMapGenerator {
  constructor(input, basePath = "") {
    // @ts-ignore
    __privateAdd(this, _generateUrls);
    // @ts-ignore
    __privateAdd(this, _urls, []);
    // @ts-ignore
    __privateAdd(this, _basePath, "");
    __privateSet(this, _basePath, basePath);
    for (const r of input) {
      const genURLs = __privateMethod(this, _generateUrls, generateUrls_fn).call(this, r.route, r.data);
      __privateGet(this, _urls).push(...genURLs);
    }
  }
  getSiteMap() {
    return this.getXML();
  }
  pushPaths(paths) {
    for (const path of paths) {
      __privateGet(this, _urls).push(`${path}`);
    }
  }
  // @ts-ignore
  get urlCount() {
    return __privateGet(this, _urls).length;
  }
  // @ts-ignore
  getXML() {
    __privateSet(this, _urls, [...new Set(__privateGet(this, _urls))]);
    const items = `	<url><loc>${__privateGet(this, _basePath)}` + __privateGet(this, _urls).join(`</loc></url>
	<url><loc>${__privateGet(this, _basePath)}`) + "</loc></url>";
    let xml = '<?xml version="1.0" encoding="UTF-8"?>\n';
    xml += '<urlset xmlns="https://www.sitemaps.org/schemas/sitemap/0.9">\n';
    xml += `${items}
`;
    xml += "</urlset>";
    return xml;
  }
}
_urls = new WeakMap();
_basePath = new WeakMap();
_generateUrls = new WeakSet();
generateUrls_fn = function(route, data) {
  const urls = [];
  if (data) {
    for (const o of data) {
      if (Array.isArray(o) && o.length > 1 && Array.isArray(o[1])) {
        urls.push(...__privateMethod(this, _generateUrls, generateUrls_fn).call(this, route.replace("@", o[0]), o[1]));
      } else if (typeof o === "string") {
        urls.push(route.replace("@", o));
      }
    }
  }
  return urls;
};

const lsDir = function(dir, files = []) {
  if (statSync(dir).isDirectory()) {
    readdirSync(dir).forEach((file) => {
      const absolute = join(dir, file);
      if (statSync(absolute).isDirectory()) {
        return lsDir(absolute, files);
      } else {
        files.push(absolute);
      }
    });
  }
  return files;
};
const module = defineNuxtModule({
  meta: {
    name: "nuxt-mapple",
    version: "0.0.1",
    configKey: "mapple",
    compatibility: { nuxt: "^3.0.0" }
  },
  defaults: {
    basePath: "",
    dynamicRoutes: [],
    staticRoutes: [],
    verbose: false,
    useContent: false,
    excludeContent: void 0
  },
  setup: function(options, nuxt) {
    let sitemapRoutes;
    let pages = [];
    let contentPaths = [];
    const resolver = createResolver(import.meta.url);
    const filePath = resolver.resolve(
      nuxt.options.srcDir,
      "node_modules/.cache/.sitemap/sitemap.xml"
    );
    if (options.useContent) {
      const dir = resolver.resolve(
        nuxt.options.srcDir,
        "content"
      );
      const contentPath = join(nuxt.options.srcDir, "content");
      const files = lsDir(dir) || [];
      contentPaths = files.filter((el) => el.match(/\/[^.]*(\.md$|[[\]:])/)).map((el) => {
        return el.replace(contentPath, "").replace(/\.md$/, "");
      });
      if (options.excludeContent) {
        contentPaths = contentPaths.filter((el) => {
          const match = options.excludeContent?.exec(el);
          return match === null;
        });
      }
    }
    nuxt.options.nitro.publicAssets = nuxt.options.nitro.publicAssets || [];
    nuxt.options.nitro.publicAssets.push({ baseURL: "/", dir: dirname(filePath) });
    nuxt.hook("pages:extend", (pagesExtend) => {
      pages = pages.concat(pagesExtend);
      const sitemapRoutesOrig = pages.map((route) => route.path);
      sitemapRoutes = sitemapRoutesOrig.filter((r) => !r.includes(":"));
      sitemapRoutes = [...sitemapRoutes, ...options?.staticRoutes || []].filter((v) => !!v);
      const generator = new SiteMapGenerator(options?.dynamicRoutes, options.basePath);
      generator.pushPaths(contentPaths);
      generator.pushPaths(sitemapRoutes);
      const sitemap = generator.getSiteMap();
      mkdirSync(dirname(filePath), { recursive: true });
      writeFileSync(filePath, sitemap);
      console.log(`\u{1F310}\uFE0F Sitemap created (${generator.urlCount} URLs${generator.urlCount === 0 ? "... did you forget your config?" : ""})`);
      if (options.verbose) {
        console.log("-----------------------");
        console.log(sitemap);
        console.log("-----------------------");
      }
    });
  }
});

export { module as default };
